
from textual.widgets import Button, Static, Checkbox
from textual.containers import Horizontal, Vertical, VerticalScroll
from textual.screen import Screen

from spotidex.cli.custom_widgets.AppInterface import AppInterface
from spotidex.cli.app_screens.DownloadInterface import Download

CSS = """
SelectDownload{
    align:center middle;


}
#interface{
    padding:1 1;
}

#all-tracks {
    border:tall grey 40%;
    width: 64;
    height: 100%;
    background: $boost;
    padding-left:1;
    padding-right:1;
    margin-left:1;
}


#selectsongs-label{
    dock:right;
    margin-right: 15;
}

#selected-tracks-buttons-container{
    dock:right;
    width: 27;
}

#deselect-all-button{

    margin-top: 1;
    width: 25;
    margin-left: 1;
}

#select-all-button{
    width: 25;
    margin-top: 1;
    margin-left: 1;
}

#download-selected-button{
    width: 25;
    margin-top: 1;
    margin-left: 1;
}

#selected-tracks-label{
    width:27;
    height: 3;
    border:tall green 40%;
    background:$boost;
    content-align:center middle;
    margin-bottom:2;

}



.all-songs-checkbox {
    width:56;
    padding-right:3;
}
.all-songs-checkbox:hover {
    text-style:none;
    opacity:0.8;
}
.all-songs-checkbox.-on{
    border: tall transparent;
    background:#32CD32;
    opacity:0.5;
}

.all-songs-checkbox:focus > .toggle--label {
    text-style: none;
    margin-left:0;
}

.all-songs-checkbox:focus {
    border:tall transparent
}

"""


class SelectDownload(Screen):
    DEFAULT_CSS = CSS
    selected_tracks = []

    def __init__(self):
        self.tracks = self.app.link_details[1]
        super().__init__()

    def compose(self):
        yield AppInterface(
            Horizontal(VerticalScroll(id="all-tracks"), id="all-tracks-container"),
            Vertical(
                Static("SELECTED TRACKS : 0", id="selected-tracks-label"),
                Button("DESELECT ALL", "error", id="deselect-all-button"),
                Button("SELECT ALL", "primary", id="select-all-button"),
                Button("DOWNLOAD SELECTED ", "success", id="download-selected-button",disabled=True),
                id="selected-tracks-buttons-container",
            ),
        )

    def on_checkbox_changed(self) -> None:
        selected_songs = []
        for checkbox in self.app.query(".all-songs-checkbox"):
            if checkbox.value == True:
                selected_songs.append(str(checkbox.label).split(". ",maxsplit=1)[1])
        self.app.query_one("#selected-tracks-label").update(
            f"SELECTED TRACKS : {len(selected_songs)}"
        )
        self.app.selected_tracks = [self.tracks[song] for song in selected_songs]
        if len(self.app.selected_tracks) != 0 :
            self.query_one('#download-selected-button').disabled = False
        else:
            self.query_one('#download-selected-button').disabled = True

    def on_mount(self):
        self.app.query_one("#nav-label").update("[bold]SELECT TRACKS TO DOWNLOAD")
        for song, track_no in zip(
            self.tracks.keys(), range(1, len(self.tracks.keys()) + 1)
        ):
            self.app.query_one("#all-tracks").mount(
                Checkbox(f"{track_no}. " + song, classes="all-songs-checkbox")
            )
        for i in self.app.query(".all-songs-checkbox"):
            i.BUTTON_INNER = ""
            i.BUTTON_LEFT = ""
            i.BUTTON_RIGHT = ""

    def on_button_pressed(self, event: Button.Pressed) -> None:
        if event.button.id == "deselect-all-button":
            for checkbox in self.app.query(".all-songs-checkbox"):
                checkbox.value = False
        elif event.button.id == "select-all-button":
            for checkbox in self.app.query(".all-songs-checkbox"):
                checkbox.value = True
        elif event.button.id == "download-selected-button":
            self.app.push_screen(Download())




from spotidex import SpotifyContent
from spotidex import SpotifyDownloader
from pathlib import Path
import os
import shutil

scraper = SpotifyContent()


app_description = """[italic]Spotidex  \
is a cutting-edge powerful CLI app designed to enhance your Spotify experience. \
With an intuitive and user-friendly interface, \
Spotidex allows you to effortlessly download your favorite Spotify songs \
by simply pasting the Spotify link. Liberate your music and take it with \
you wherever you go, even when offline.

Spotidex supports a wide range of Spotify links, enabling \
users to download playlists, individual tracks, and entire albums effortlessly.

App's GitHub repository is available at [link=https://github.com/libin-codes/spotidex]https://github.com/libin-codes/spotidex[/link][/italic]
"""

def get_link_details(link):
    if "track" in link:
        track_details = scraper.track_details(link)
        return (
            {
                "TRACK ID": link.split("/")[-1].split("?")[0],
                "TRACK NAME": track_details["track_name"],
                "ARTIST": track_details["artist"],
                "ALBUM NAME": track_details["album"],
            },
        )

    elif "playlist" in link:
        playlist_details = scraper.playlist_details(link)
        return (
            {
                "PLAYLIST ID": link.split("/")[-1].split("?")[0],
                "PLAYLIST NAME": playlist_details["playlist_name"],
                "PLAYLIST CREATOR": playlist_details["creator"],
                "TOTAL TRACKS": str(playlist_details["total_tracks"]),
            },
            {
                track["track_name"]: track["link"]
                for track in playlist_details["track_details"]
            },
        )
    elif "album" in link:
        album_details = scraper.album_details(link)
        return (
            {
                "ALBUM ID": link.split("/")[-1].split("?")[0],
                "ALBUM NAME": album_details["album_name"],
                "RELEASE DATE": album_details["release_date"],
                "TOTAL TRACKS": str(album_details["total_tracks"]),
            },
            {
                track["track_name"]: track["link"]
                for track in album_details["track_details"]
            },
        )


def shorten_path(path: str, length):
    while len(path) > length:
        path_parts = path.split("\\")
        if "..." in path_parts:
            path_parts.pop(2)
        else:
            path_parts.pop(1)
            path_parts.insert(1, "...")
        path = "\\".join(path_parts)
    return path


def create_download_path(path, folder_name):
    folder_path = Path(path, folder_name)
    folder_path.mkdir(parents=True, exist_ok=True)

    return str(folder_path)


input_validator = lambda value: (
    True
    if ("spotify" in value)
    and (any(keyword in value for keyword in ["track", "playlist", "album"]))
    else False
)

def get_data_drive_path():
    if os.name == 'nt':  # Windows
        system_drive = os.environ['SYSTEMDRIVE'] if 'SYSTEMDRIVE' in os.environ else 'C:'
        drives = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z']
        available_drives = [drive + ':\\' for drive in drives if os.path.exists(drive + ':\\') and drive + ':' != system_drive]
        return available_drives[0] if available_drives else None
    else:  # Unix-like systems
        # Filter out root directory ('/') from mounted drives
        mounted_drives = [root for root, dirs, files in os.walk('/') if dirs or files]
        return mounted_drives[0] if mounted_drives else None

def get_downloads_folder():
    # Get the user's home directory
    home_dir = os.path.expanduser("~")
    
    # Operating system specific logic to get the Downloads folder
    if os.name == 'posix':  # Unix-like OS (Linux, macOS)
        downloads_folder = os.path.join(home_dir, 'Downloads')
    elif os.name == 'nt':   # Windows
        downloads_folder = os.path.join(home_dir, 'Downloads')
    
    return downloads_folder




from textual.widgets import Static, RichLog, Label, ProgressBar
from textual.containers import Horizontal
from textual.screen import Screen
from textual.app import App

from rich.table import Table
from rich.align import Align

from spotidex.cli.utils import Path,SpotifyDownloader

from spotidex.cli.custom_widgets.AppInterface import AppInterface
from spotidex.cli.utils import shorten_path
from textual import work
from textual.worker import Worker, get_current_worker
from spotidex.cli.popup_screens.NetworkErrorPopup import NetworkErrorScreen

CSS = """

Download{
    align:center middle;

}

#download-interface #interface{
    padding: 1 3;
    grid-rows:1fr 5;
    grid-gutter: 1 1;
}

#music-info{
    width:95;
   
}


#info-container{
    border: heavy ansi_bright_green;
    border-title-align: center;
    content-align:center middle;
    padding-left:1;
 
}

#progress-bar-container{
    width:90;
    height:5;
    background:$boost;
    padding-right:1;

    
}
#progress{
    width:16;
    height:5;
    background:$boost;
    content-align:center middle;
    text-style: bold;
}



#bc{
    height:3;
    margin-top:1;
    margin-left:3;
    margin-right:2;
    padding-top:1;
    padding-left:4;
    background:$boost;
}

#stat{
    width:14;
}
"""


class Download(Screen):
    DEFAULT_CSS = CSS

    def compose(self):
        yield AppInterface(
            Horizontal(RichLog(id="music-info"), id="info-container"),
            Horizontal(
                Label("PROGRESS", id="progress"),
                Horizontal(
                    Static("DOWNLOADING : ", id="stat"), ProgressBar(id="bar"), id="bc"
                ),
                id="progress-bar-container",
            ),
            id="download-interface",
        )

    def on_mount(self):
        self.app.query_one("#nav-label").update("[bold]DOWNLOAD INTERFACE")
        text_log = self.app.query_one("#music-info")
        text_log.clear()

        table = Table(
            show_header=False, row_styles=["yellow"] * 3, width=85, show_lines=True
        )
        if "track" in self.app.spotify_link:
            link_type = "TRACK"
        elif "playlist" in self.app.spotify_link:
            link_type = "PLAYLIST"
        elif "album" in self.app.spotify_link:
            link_type = "ALBUM"

        self.query_one("#info-container").border_title = f"{link_type} INFO"
        name = self.app.link_details[0][f"{link_type} NAME"]

        if "track" in self.app.spotify_link:
            full_track_info = f"{name} by {self.app.link_details[0]['ARTIST']}"
            track_info = (
                full_track_info[:57] + "..."
                if len(full_track_info) > 57
                else (name[:57] + "..." if len(name) > 40 else full_track_info)
            )
        else:
            track_info = (
                f"{name[:45]}..."
                if len(name) > 45
                else f"{name} [{len(self.app.selected_tracks)} Tracks]"
            )

        download_info = {
            f"{link_type.capitalize()} Info": track_info,
            "Download Path": shorten_path(self.app.saved_settings["download_path"], 57),
            "Download Quality": self.app.saved_settings["download_quality"],
            "Include Metadata": str(self.app.saved_settings["add_metadata"]),
        }

        for key, value in download_info.items():
            table.add_row(Align(key, align="center"), Align(value, align="center"))

        text_log.write(table)
        self.app.query_one("#back-button").label = "GO HOME"
        self.app.query_one("#back-button").disabled = True
        self.app.query_one("#exit-button").disabled = True
        self.download_music()

    def progress_hook(self, percent):
        if self.app.query_one("ProgressBar").percentage != 0.0:
            self.app.query_one("ProgressBar").update(total=100)
        self.app.query_one("ProgressBar",ProgressBar).update(progress=float(percent))

    def on_worker_state_changed(self, event) -> None:
        def eval(value):
            if value == "retry":
                self.app.query_one("ProgressBar",ProgressBar).update(progress=0, total=None)
                self.download_music()

        if str(event.state) == "WorkerState.CANCELLED":
            self.app.push_screen(NetworkErrorScreen(), eval)

    @work(thread=True, exclusive=True)
    def download_music(self):
        try:
            if "track" in self.app.spotify_link:
                SpotifyDownloader().download_track(
                    self.app.spotify_link,
                    self.progress_hook,
                    self.app.saved_settings["download_path"],
                    self.app.saved_settings["download_quality"],
                    self.app.saved_settings["add_metadata"],
                )
            elif self.app.selected_tracks == self.app.link_details[1].keys():
                if "playlist" in self.app.spotify_link:
                    SpotifyDownloader().download_playlist(
                        self.app.spotify_link,
                        self.progress_hook,
                        self.app.saved_settings["download_path"],
                        self.app.saved_settings["download_quality"],
                        self.app.saved_settings["add_metadata"],
                    )
                elif "album" in self.app.spotify_link:
                    SpotifyDownloader().download_album(
                        self.app.spotify_link,
                        self.progress_hook,
                        self.app.saved_settings["download_path"],
                        self.app.saved_settings["download_quality"],
                        self.app.saved_settings["add_metadata"],
                    )
            else:
                if "playlist" in self.app.spotify_link:
                    folder_name = self.app.link_details[0]["PLAYLIST NAME"]
                elif "album" in self.app.spotify_link:
                    folder_name = self.app.link_details[0]["ALBUM NAME"]
                SpotifyDownloader().download_multiple_songs(
                    self.app.selected_tracks,
                    f"{folder_name} [{len(self.app.selected_tracks)} Tracks]",
                    self.progress_hook,
                    self.app.saved_settings["download_path"],
                    self.app.saved_settings["download_quality"],
                    self.app.saved_settings["add_metadata"],
                )
            self.app.query_one("#stat").update("DOWNLOADED : ")
            self.app.query_one("#back-button").disabled = False
            self.app.query_one("#exit-button").disabled = False
        except Exception:
            get_current_worker().cancel()



FFMPEG_URLS = {
    "windows": {
        "amd64": "https://github.com/eugeneware/ffmpeg-static/releases/download/b4.4/win32-x64",
        "i686": "https://github.com/eugeneware/ffmpeg-static/releases/download/b4.4/win32-ia32",
    },
    "linux": {
        "x86_64": "https://github.com/eugeneware/ffmpeg-static/releases/download/b4.4/linux-x64",
        "x86": "https://github.com/eugeneware/ffmpeg-static/releases/download/b4.4/linux-ia32",
        "arm32": "https://github.com/eugeneware/ffmpeg-static/releases/download/b4.4/linux-arm",
        "aarch64": "https://github.com/eugeneware/ffmpeg-static/releases/download/b4.4/linux-arm64",
    },
    "darwin": {
        "x86_64": "https://github.com/eugeneware/ffmpeg-static/releases/download/b4.4/darwin-x64",
        "arm": "https://github.com/eugeneware/ffmpeg-static/releases/download/b4.4/darwin-arm64",
    },
}

client_id     = "df3b20346d3a47a49295015021776d5d"
client_secret = "69047630688b498fa601fc311ef19577"
bitrate       = {"low":'24','medium':'128','high':'320'} 


from textual.widgets import Button
from textual.containers import Horizontal
from textual.screen import Screen
from textual.reactive import reactive

from spotidex.cli.custom_widgets.DownloadPathSelector import DownloadPathSelector
from spotidex.cli.custom_widgets.MetadataCheckBox import MetadataCheckBox
from spotidex.cli.custom_widgets.DownloadQualitySelection import DownloadQualitySelection
from spotidex.cli.custom_widgets.AppInterface import AppInterface
from spotidex.cli.utils import Path

CSS = """
Settings{
    align:center middle;
    height:auto;
    width:auto;

}

#settings-interface #interface{
    grid-size:2 3;
    padding:1 4;
    grid-gutter:1 2;
    align:center middle;
}
DownloadPathSelector{
    column-span:2;
}
DownloadQualitySelection{
    dock:right;
}

#metadata-download-quality-container{
    column-span:2;
}
#save-settings-button{
    width:100%;
}
#reset-settings-button{
    width:100%;
    
}
"""


class Settings(Screen):
    DEFAULT_CSS = CSS

    def compose(self):
        yield AppInterface(
            DownloadPathSelector(),
            Horizontal(
                MetadataCheckBox(),
                DownloadQualitySelection(),
                id="metadata-download-quality-container",
            ),
            Button("SAVE SETTINGS", "success", id="save-settings-button"),
            Button("RESET SETTINGS", "error", id="reset-settings-button"),
            id = "settings-interface"
        )

    def on_button_pressed(self, event: Button.Pressed) -> None:
        if event.button.id == "reset-settings-button":
            self.app.query_one(MetadataCheckBox).value = True
            self.app.query_one(DownloadPathSelector).value = str(Path.cwd())
            self.app.query_one(DownloadQualitySelection).value = "high"
        elif event.button.id == "save-settings-button":
            self.app.notify("SETTINGS SAVED SUCCESSFULLY", title="Spotidex")
            self.app.saved_settings = {
                "add_metadata":self.app.query_one(MetadataCheckBox).value ,
                "download_path": self.app.query_one(DownloadPathSelector).value,
                "download_quality": self.app.query_one(DownloadQualitySelection).value,
                }
            self.app.pop_screen()
            
    def on_mount(self):
        self.app.query_one("#nav-label").update('[bold]SETTINGS')




from spotidex.src.utils import *
from spotidex.src.static import *

from pathlib import Path

from tempfile import TemporaryDirectory
import shutil
import yt_dlp.YoutubeDL
import spotipy
from spotipy.oauth2 import SpotifyClientCredentials
from spotipy import SpotifyException
from requests import ConnectionError,ReadTimeout



class SpotifyDownloader:
    def __init__(self) -> None:
        self.spotidex_path = get_spotidex_data_directory()
        self.ffmpeg_path = get_ffmpeg()
        self.total_tracks = None
        self.sp = spotipy.Spotify(
            client_credentials_manager=
                SpotifyClientCredentials(
                    client_id,
                    client_secret,
            )
                )
        
    def _download_with_temp_directory(
        self,
        links,
        folder_name,
        custom_hook=None,
        download_path=Path.cwd(),
        quality="high",
        metadata=True,
        make_folder=True,
    ):
        with TemporaryDirectory(dir=str(self.spotidex_path)) as temp_folder:
            if make_folder:
                folder_name = get_valid_name(download_path, folder_name)
                temp_path = Path(temp_folder).joinpath(folder_name)
            else:
                temp_path = Path(temp_folder)

            done, not_done = pool_download(
                self, list(set(links)), custom_hook, temp_path, quality, metadata
            )
            if len(not_done) != 0:
                raise SpotidexError("NetworkError")
            progress_hook(self, {"status": "downloaded"}, custom_hook)
            for file_path in Path(temp_folder).iterdir():
                shutil.move(str(file_path), str(download_path))
            progress_hook(self, {"status": "transfered"}, custom_hook)
        

    def download_track(
        self,
        link,
        custom_hook=None,
        download_path=Path.cwd(),
        quality="high",
        metadata=True,
    ):
        try:
            data = self.sp.track(link.split("/")[-1].split("?")[0])
        except SpotifyException:
            raise SpotidexError("InvalidSpotifyLink")
        except (ConnectionError,ReadTimeout):
            raise SpotidexError("NetworkError")

        
        track_artist = ",".join([artist["name"] for artist in data["artists"]])
        query = data["name"]+f" song by {track_artist} official lyrics "
        file_name = get_valid_name(download_path,data["name"]+".mp3")
    
        with TemporaryDirectory(dir=str(self.spotidex_path)) as temp_folder:
            options = {
                "format": "bestaudio/best",
                "outtmpl": str(download_path)+'\\'+file_name,
                "quiet": True,
                "noprogress": True,
                "progress": "false",
                "ffmpeg_location": str(self.ffmpeg_path),
                "postprocessors": [
                    {
                        "key": "FFmpegExtractAudio",
                        "preferredcodec": "mp3",
                        "preferredquality": bitrate[quality],
                    }
                ],
                "retries": 20,
                "progress_hooks": [lambda d: progress_hook(self, d, custom_hook)]
                if custom_hook != None
                else [],
            }
                
            if self.total_tracks == None:
                options['outtmpl'] = str(Path(temp_folder))+'\\'+file_name
            ydl = yt_dlp.YoutubeDL(options)
            yt_video_result = ydl.extract_info(f"ytsearch:{query}", download=False)
            first_yt_video_link = yt_video_result["entries"][0]["webpage_url"]
            yt_video_download = ydl.extract_info(first_yt_video_link, download=True)
            downloaded_path = ydl.prepare_filename(yt_video_download)

            if metadata:
                add_metadata(data, Path(downloaded_path + ".mp3").resolve())

            if self.total_tracks == None:
                progress_hook(self, {"status": "downloaded"}, custom_hook)
                for file_path in Path(temp_folder).iterdir():
                    shutil.move(str(file_path), str(download_path))
                progress_hook(self, {"status": "transfered"}, custom_hook)
    
    def download_playlist(
        self,
        link,
        custom_hook=None,
        download_path=Path.cwd(),
        quality="high",
        metadata=True,
        make_folder=True,
    ):
        playlist_id = link.split("/")[-1].split("?")[0]
        try:
            playlist_data = self.sp.playlist(playlist_id)
            track_links = extract_track_links(self.sp, link)
        except SpotifyException:
            raise SpotidexError("InvalidSpotifyLink")
        except (ConnectionError,ReadTimeout):
            raise SpotidexError("NetworkError")
        
        folder_name = get_valid_name(download_path,playlist_data['name'])

        self._download_with_temp_directory(
                track_links,
                folder_name,
                custom_hook,
                download_path,
                quality,
                metadata,
                make_folder,
            )
            
    def download_album(
        self,
        link,
        custom_hook=None,
        download_path=Path.cwd(),
        quality="high",
        metadata=True,
        make_folder=True,
    ):
        album_id = link.split("/")[-1].split("?")[0]
        try:
            album_data = self.sp.album(album_id)
            track_links = extract_track_links(self.sp, link)
        except SpotifyException:
            raise SpotidexError("InvalidSpotifyLink")
        except (ConnectionError,ReadTimeout):
            raise SpotidexError("NetworkError")
        
        
        folder_name = get_valid_name(download_path,album_data['name'])
        
        self._download_with_temp_directory(
                track_links,
                folder_name,
                custom_hook,
                download_path,
                quality,
                metadata,
                make_folder,
            )
            
    def download_multiple_songs(
        self,
        links,
        folder_name,
        custom_hook=None,
        download_path=Path.cwd(),
        quality="high",
        metadata=True,
        make_folder=True,
    ):
        self._download_with_temp_directory(
                links,
                folder_name,
                custom_hook,
                download_path,
                quality,
                metadata,
                make_folder,
            )
            



from textual.app import ComposeResult
from textual.containers import Container
from textual.events import Mount
from textual.widgets import Label, Button, Static, Input,Header,Footer
from textual.widget import Widget
from textual.containers import Grid
from textual.reactive import reactive

from spotidex.cli.popup_screens.GoBackSettingsPopup import GoBackSettings
from spotidex.cli.popup_screens.ExitScreenPopup import ExitScreen


CSS = """
AppInterface{
    height:auto;
    width:auto;

}

#interface {
    background:$boost;
    height:19 ;
    width:95 ;
    margin-top: 1;
  
}
#loading-interface{
    background:$boost;
    height:19 ;
    width:95 ;
    margin-top: 1;
}

#app-container{
    width:99;
    height:27;
    background: $background 90% ;
    padding:1 2;
}

#appheader-container{
    layout:horizontal;
    background:$boost;
    height:5;
    width:95 ;
    padding: 1;
}

#nav-label-container{
    align:center middle;

}

#nav-label{
    color:gainsboro;
}

#back-button{
    dock:left;
    margin-left:2;
}

#exit-button{
    dock:right;
    margin-right:2;
}

"""


class AppHeader(Static):
    DEFAULT_CSS = CSS

    def compose(self):
        with Container(id="appheader-container"):
            yield Button("GO BACK", id="back-button")
            with Container(id="nav-label-container"):
                yield Label(id="nav-label")
            yield Button("EXIT", id="exit-button")

    def eval_settings(self, value):
        settings = self.app.saved_settings
        if value:
            self.app.notify("SETTINGS SAVED SUCCESSFULLY", title="Spotidex")
            self.app.saved_settings = {
                "add_metadata": self.app.query_one("MetadataCheckBox").value,
                "download_path": self.app.query_one("DownloadPathSelector").value,
                "download_quality": self.app.query_one(
                    "DownloadQualitySelection"
                ).value,
            }
        else:
            self.app.query_one("MetadataCheckBox").value = settings["add_metadata"]
            self.app.query_one("DownloadPathSelector").value = settings["download_path"]
            self.app.query_one("DownloadQualitySelection").value = settings[
                "download_quality"
            ]
        self.app.pop_screen()

    def on_button_pressed(self, event: Button.Pressed) -> None:

        if event.button.id == "exit-button":
            self.app.push_screen(ExitScreen())

        elif event.button.id == "back-button":
            if str(self.app.screen) == "Settings()":
                settings = self.app.saved_settings
                if (
                    (
                        self.app.query_one("MetadataCheckBox").value
                        != settings["add_metadata"]
                    )
                    or (
                        self.app.query_one("DownloadPathSelector").value
                        != settings["download_path"]
                    )
                    or (
                        self.app.query_one("DownloadQualitySelection").value
                        != settings["download_quality"]
                    )
                ):
                    self.app.push_screen(GoBackSettings(), self.eval_settings)
                else:
                    self.app.pop_screen()
            elif str(self.app.screen) == "MainMenuInterface()":
                self.app.pop_screen()
                self.app.query_one("#search-input").value = ""
                self.app.uninstall_screen("SelectDownloadScreen")

            elif str(self.app.screen) == "Download()":

                for _ in range(len(self.app.screen_stack) - 2):
                    self.app.pop_screen()
                self.app.query_one("#search-input", Input).clear()
                self.app.uninstall_screen("SelectDownloadScreen")

            else:
                self.app.pop_screen()


class AppInterface(Widget):
    DEFAULT_CSS = CSS

    def __init__(self, *children, disabled: bool = False, id=None) -> None:
        self.childrens = children
        super().__init__(disabled=disabled, id=id)

    def compose(self):
     
        with Container(id="app-container"):
            yield AppHeader()
            yield Grid(*self.childrens, id="interface")

    def on_mount(self):
        self.app.mount(Header(show_clock=True))
        self.app.mount(Footer())



from textual.widgets import Input, Button, Label
from textual.screen import Screen
from textual.containers import Horizontal

from spotidex.cli.custom_widgets.AppInterface import AppInterface
from spotidex.cli.app_screens.MainMenuInterface import MainMenuInterface
from spotidex.cli.utils import input_validator, app_description
from spotidex.cli.utils import get_link_details

from textual.worker import Worker, get_current_worker
from textual import work

CSS = """
Search {
    align: center middle;
}

#search-interface #interface{
    grid-size:1 2;
    grid-rows:3 1fr;
    grid-gutter:1;
    padding: 2 3 0 3;
    
}

#search-input{
    column-span:1;
    width: 74%;
}

#search-button{
    column-span:1;
    width: 23%;
    margin-left:1;
}

#app-description{
    border: round ansi_bright_cyan;
    border-title-align: center;

}

#search-input:blur{
    border:tall black
}
"""


class Search(Screen):
    DEFAULT_CSS = CSS

    def compose(self):
        yield AppInterface(
            Horizontal(
                Input(
                    id="search-input",
                    placeholder="PASTE YOUR SPOTIFY LINK HERE",
                ),
                Button("SEARCH", "success", id="search-button", disabled=True),
            ),
            Label(app_description, id="app-description", shrink=True),
            id="search-interface",
        )

    @work(exclusive=True, thread=True)
    def get_link_info(self):
        self.app.query_one("LoadingIndicator").styles.background = "#1c1c1c"
        try:
            self.app.link_details = get_link_details(self.app.spotify_link)
        except Exception as spotidex_error:
            if spotidex_error.message == "NetworkError":
                self.app.notify(
                    "Please check you internet connection and try again",
                    title="Network Error",
                    severity="error",
                )
            elif spotidex_error.message == "InvalidSpotifyLink":
                self.app.notify(
                    "The link provided is not valid, please check the link and try again",
                    title="INVALID LINK",
                    severity="error",
                )
            get_current_worker().cancel()
        self.app.query_one("#interface").loading = False
        self.app.query_one("#exit-button").disabled = False
        self.app.query_one("#interface").styles.padding = (2, 3, 0, 3)

    def on_worker_state_changed(self, event: Worker.StateChanged) -> None:
        if str(event.state) == "WorkerState.CANCELLED":
            self.query_one("#search-input").visible = True
            self.query_one("#app-description").visible = True
            self.query_one("#search-button").visible = True
        elif str(event.state) == "WorkerState.SUCCESS":
            self.app.push_screen(MainMenuInterface())
            self.query_one("#search-input").visible = True
            self.query_one("#app-description").visible = True
            self.query_one("#search-button").visible = True
            self.query_one("#search-button").disabled = True

    def on_mount(self):
        self.app.query_one("#nav-label").update("[bold]HOME")
        self.query_one("#app-description").border_title = "ABOUT SPOTIDEX"
        self.app.query_one("#back-button").disabled = True

    def on_button_pressed(self, event: Button.Pressed) -> None:
        if event.button.id == "search-button":
            self.app.query_one("#interface").styles.padding = 0
            self.app.spotify_link = self.query_one("#search-input").value
            self.app.query_one("#interface").loading = True
            self.app.query_one("#exit-button").disabled = True
            self.query_one("#search-input").visible = False
            self.query_one("#app-description").visible = False
            self.query_one("#search-button").visible = False
            self.get_link_info()

    def on_input_changed(self, event: Input.Changed):
        if input_validator(event.input.value):
            self.app.query_one("#search-input", Input).styles.border = ("tall", "green")
            self.app.query_one("#search-button").disabled = False
        elif event.input.value == "":
            self.app.query_one("#search-input", Input).styles.border = ("tall", "black")
        else:
            self.app.query_one("#search-input", Input).styles.border = ("tall", "red")
            self.app.query_one("#search-button").disabled = True



import os,re,stat,platform,requests,subprocess
from mutagen.id3 import ID3, APIC, TPE1, TIT2, TALB
import concurrent.futures
from pathlib import Path
from tqdm import tqdm

from spotidex.src.static import FFMPEG_URLS

progress_dict = {}
class SpotidexError(Exception):
    def __init__(self,message):
        self.message = message
        super().__init__(message)

def extract_track_links(sp, link):
    id = link.split("/")[-1].split("?")[0]
    is_playlist = "playlist" in link
    data = sp.playlist(id) if is_playlist else sp.album(id)
    total_tracks = data["tracks"]["total"]
    offset, limit = 0, 100 if is_playlist else 40
    track_links = []

    while offset < total_tracks:
        results = (
            sp.playlist_items(id, offset=offset, limit=limit)
            if is_playlist
            else sp.album_tracks(id, offset=offset, limit=limit)
        )
        for track in results["items"]:
            track_links.append(track["track"]["href"] if is_playlist else track["href"])
        offset += limit
    return track_links


def add_metadata(data, path):
    audio = ID3(path)
    audio.add(TIT2(encoding=3, text=data["name"]))
    audio.add(
        TPE1(encoding=3, text=",".join([artist["name"] for artist in data["artists"]]))
    )
    audio.add(TALB(encoding=3, text=data["album"]["name"]))
    audio.add(
        APIC(
            encoding=3,
            mime="image/jpeg",
            type=3,
            desc="Cover",
            data=requests.get(data["album"]["images"][0]["url"]).content,
        )
    )

    audio.save(v2_version=3)


def pool_download(
    downloader,
    links,
    custom_hook,
    download_path,
    quality,
    metadata,
    max_concurrent=4,
):
    downloader.total_tracks = len(links)
    with concurrent.futures.ThreadPoolExecutor(max_concurrent) as executor:
        tasks = [
            executor.submit(
                downloader.download_track,
                link,
                custom_hook,
                download_path,
                quality,
                metadata,
            )
            for link in links
        ]
        return concurrent.futures.wait(tasks)


def progress_hook(self, d, custom_hook=None):
    global progress_dict
    if custom_hook is None:
        return
    if d["status"] == "downloading":
        percent = "".join(
            char for char in d["_percent_str"] if char.isdigit() or char == "."
        )
        if self.total_tracks != None:
            progress_dict.update(
                {d["info_dict"]["id"]: float(percent[3:]) * 0.943}
            )
            progress = sum(progress_dict.values()) / self.total_tracks
            custom_hook(str(round(progress, 1)))
        else:
            custom_hook(str(round(float(percent[3:]) * 0.943, 1)))
    elif d["status"] == "downloaded":
        custom_hook("97.8")
    elif d["status"] == "transfered":
        custom_hook("100")
        progress_dict = {}
        self.total_tracks = None


def parallel_searches(self, track_links):
    with concurrent.futures.ThreadPoolExecutor(max_workers=6) as executor:
        tasks = [executor.submit(self.track_details, link) for link in track_links]
        concurrent.futures.wait(tasks)
    return [task.result() for task in tasks]


def get_spotidex_data_directory():
    spotidex_path = Path(os.path.expanduser("~"), ".spotidex")
    spotidex_path.mkdir(parents=True, exist_ok=True)
    return spotidex_path

def get_ffmpeg():
    os_name = platform.system().lower()
    os_arch = platform.machine().lower()

    ffmpeg_url = FFMPEG_URLS.get(os_name, {}).get(os_arch)
    ffmpeg_path = get_ffmpeg_path(os_name)

    if ffmpeg_path.exists():
        run_ffmpeg(ffmpeg_path)
        return ffmpeg_path

    download_ffmpeg(ffmpeg_url, ffmpeg_path, os_name)
    return ffmpeg_path

def get_ffmpeg_path(os_name):
    executable_extension = ".exe" if os_name == "windows" else ""
    return get_spotidex_data_directory() / f"ffmpeg{executable_extension}"

def run_ffmpeg(ffmpeg_path):
    current_path = Path().cwd()
    os.chdir(get_spotidex_data_directory())
    try:
        subprocess.run(
            [ffmpeg_path], stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL
        )
    except Exception:
        print("Network Error: Please check your internet connection and try later")
        exit()
    finally:
        os.chdir(current_path)


def download_ffmpeg(ffmpeg_url, ffmpeg_path, os_name):
    try:
        ffmpeg_binary = requests.get(ffmpeg_url, stream=True, timeout=10)
    except Exception:
        print("Network Error: Please check your internet connection and try later")
        exit()

    with open(ffmpeg_path, "wb") as ffmpeg_file, tqdm(
        total=100,
        unit="%",
        unit_scale=True,
        unit_divisor=1024,
        dynamic_ncols=True,
        bar_format="Initializing : |{bar:50}| {percentage:.0f}% ",
    ) as progress_bar:
        try:
            for data in ffmpeg_binary.iter_content(chunk_size=1024):
                ffmpeg_file.write(data)
                progress_bar.update(
                    len(data)
                    * 100
                    / int(ffmpeg_binary.headers.get("content-length", 0))
                )
        except Exception:
            progress_bar.close()
            print("Network Error: Please check your internet connection and try later")
            exit()

    if os_name in ["linux", "darwin"]:
        ffmpeg_path.chmod(ffmpeg_path.stat().st_mode | stat.S_IEXEC)

def get_valid_name(path, name):
    name, i = re.sub(r"[^\w\d(),.\-\[\] ]", "", name), 1
    add_mp3 = ".mp3" if ".mp3" in name else ""
    while Path(path, name).exists():
        name, i = name.replace('.mp3',"").split(" (")[0] + f" ({i})" + add_mp3, i + 1
    return name.replace('.mp3',"")




from textual.widgets import Button, Static
from textual.containers import Container
from textual.widget import Widget

from spotidex.cli.popup_screens.DownloadPathPopup import DownloadPathScreen
from spotidex.cli.utils import Path, shorten_path
from textual.reactive import reactive

CSS = """
DownloadPathSelector{
    width:auto;
    height:auto;

}

#download-path-container{
    layout:horizontal;
    width:87;
    height: auto;
    padding:1;
    background:$boost;
}


#download-path-button{
    min-width:12;
    margin-left:1;
}

#download-path-label{
    background:$boost;
    width:71;
    height:3;
    border:tall $primary;
    padding-left:1;
}

"""


class DownloadPathSelector(Widget):
    DEFAULT_CSS = CSS
    value = reactive(str(Path.cwd()))

    def compose(self):
        with Container(id="download-path-container"):
            yield Static(id="download-path-label")
            yield Button("CHANGE", id="download-path-button", variant="primary")

    def on_button_pressed(self, event: Button.Pressed) -> None:
        def change_value(value):
            self.value = value

        if event.button.id == "download-path-button":
            self.app.push_screen(DownloadPathScreen(), change_value)

    def watch_value(self, value):
        self.query_one("#download-path-label").update("[bold]"+shorten_path(value, 65))



from textual.app import App
from spotidex.cli.app_screens.SearchInterface import Search
from spotidex.cli.app_screens.SettingsInterface import Settings
from spotidex.cli.utils import Path


class SpotidexApp(App):
    TITLE = "SPOTIDEX"
    SUB_TITLE = "SPOTIFY DOWNLOADER"
    SCREENS = {"SettingsScreen": Settings}

    def on_ready(self) -> None:
        self.app.saved_settings = {
            "add_metadata": True,
            "download_path": str(Path.cwd()),
            "download_quality": "high",
        }
        self.push_screen(Search())
 
def main():
    SpotidexApp().run()






from spotidex.src.utils import *
from spotidex.src.static import *
from spotipy import Spotify
from spotipy.oauth2 import SpotifyClientCredentials
from spotipy import SpotifyException
from requests import ConnectionError,ReadTimeout


class SpotifyContent:
    def __init__(self) -> None:
        client_cred_manager = SpotifyClientCredentials(client_id, client_secret)
        self.sp = Spotify(client_credentials_manager=client_cred_manager)

    def album_details(self, link):
        album_id = link.split("/")[-1].split("?")[0]
        try:
            album_data = self.sp.album(album_id)
            track_links = extract_track_links(self.sp,link)
        except (ConnectionError,ReadTimeout):
            raise SpotidexError("NetworkError")
        except SpotifyException:
            raise SpotidexError("InvalidSpotifyLink")
        

        details = {
            "album_name": album_data["name"],
            "album_type": album_data["album_type"],
            "total_tracks": album_data["tracks"]["total"],
            "release_date": album_data["release_date"],
            "label": album_data["label"],
            "image_url": album_data["images"][0]["url"],
            "track_details":  parallel_searches(self,track_links),
        }

        return details

    def playlist_details(self, link):
        playlist_id = link.split("/")[-1].split("?")[0]
        try:
            playlist_data = self.sp.playlist(playlist_id)
            track_links = extract_track_links(self.sp, link)
        except (ConnectionError,ReadTimeout):
            raise SpotidexError("NetworkError")
        except SpotifyException:
            raise SpotidexError("InvalidSpotifyLink")
        
        

        details = {
            "playlist_name": playlist_data["name"],
            "description": playlist_data["description"],
            "creator": playlist_data["owner"]["display_name"],
            "followers": playlist_data["followers"]["total"],
            "total_tracks": playlist_data["tracks"]["total"],
            "image_url": playlist_data["images"][0]["url"],
            "track_details": parallel_searches(self,track_links),
        }

        return details

    def track_details(self, link):
        try:
            track_data = self.sp.track(link.split("/")[-1].split("?")[0])
        except (ConnectionError,ReadTimeout):
            raise SpotidexError("NetworkError")
        except SpotifyException:
            raise SpotidexError("InvalidSpotifyLink")
        details = {
            "track_name": track_data["name"],
            "track_number": track_data["track_number"],
            "disc_number": track_data["disc_number"],
            "album": track_data["album"]["name"],
            "artist": ", ".join([artist["name"] for artist in track_data["artists"]]),
            "cover_image_url": track_data["album"]["images"][0]["url"],
            "link": track_data["external_urls"]["spotify"],
        }

        return details



from textual.screen import ModalScreen
from textual.widgets import Label,Button
from textual.containers import Grid

CSS ='''

ExitScreen {
    align: center middle;
}

#exit-dialog-box{
    grid-size: 2;
    grid-gutter: 1 2;
    grid-rows: 1fr 3;
    padding: 0 1;
    width: 50;
    height: 11;
    border: thick $background 80%;
    background: $surface;
}

#exit-question {
    column-span: 2;
    height: 1fr;
    width: 1fr;
    content-align: center middle;
}

#yes-exit-button {
    width: 100%;
}

#exit-no-button {
    width: 100%;
}

'''

class ExitScreen(ModalScreen):
    DEFAULT_CSS = CSS
    def compose(self):
        yield Grid(
            Label("[bold]ARE YOU SURE YOU WANT TO EXIT?", id="exit-question"),
            Button("YES", id="yes-exit-button",variant='success'),
            Button("NO", variant="error", id="exit-no-button"),
            id="exit-dialog-box",
        )

    def on_button_pressed(self, event: Button.Pressed) -> None:
        if event.button.id == "yes-exit-button":
            self.app.exit()
        else:
            self.dismiss()


from textual.screen import ModalScreen
from textual.widgets import DirectoryTree, RichLog, Button
from textual.containers import Container

from spotidex.cli.utils import (
    shorten_path,
    Path,
    get_data_drive_path,
    get_downloads_folder,
)

CSS = """
DownloadPathScreen{
  
    align:center middle;

}
DirectoryTree{
    background:transparent;
}

#downloadpath-main-container{
    padding: 0 1;
    width: 75;
    height: 22;
    border: thick $background 80%;
    background: $surface;

}

#select-download-path-container{
    width:71;
    height:13;
    background:$boost;
    padding:1;

}

#download-path-log{
    background:transparent;
    width:75;
}

#download-path-log-container{
    padding-left:2;
    width:63;
    height:3;
    border:grey;
}

#download-path-button-container{
    layout:horizontal;
    height:3;
    width:71;
    margin-top:1;
    align:center middle;
}

#download-path-confirm-button{
    width:23;
    dock:left;

}

#download-path-reset-button{
    width:22;
    
}

#download-path-back-button{
    
    height:3;
    width:4;
    min-width:8;
    dock:right;
    border:gray;
    background:transparent;

}

#downloads-folder-button{
    dock:right;
    width:22;
}

#download-path-back-button:hover {
   tint: $background 20%;
}

"""


class DownloadPathScreen(ModalScreen):
    DEFAULT_CSS = CSS
    current_path = str(Path(get_data_drive_path()))

    class FilteredDirectoryTree(DirectoryTree):
        def filter_paths(self, paths):
            self.show_root = False
            self.show_guides = False
            self.guide_depth = 3
            return [path for path in paths if path.is_dir()]

    def compose(self):
        with Container(id="downloadpath-main-container"):
            with Container(id="download-path-log-container"):
                yield RichLog(
                    highlight=True, markup=True, id="download-path-log"
                ).write(shorten_path(self.current_path, 60))
            yield Button("BACK", id="download-path-back-button", variant="primary")
            with Container(id="select-download-path-container"):
                yield self.FilteredDirectoryTree(self.current_path, id="filteredtree")
            with Container(id="download-path-button-container"):
                yield Button(
                    "CONFIRM", id="download-path-confirm-button", variant="success"
                )
                yield Button(
                    "CURRENT DIRECTORY",
                    id="download-path-reset-button",
                    variant="error",
                )
                yield Button(
                    "DOWNLOADS FOLDER", id="downloads-folder-button", variant="warning"
                )

    def refresh_tree(self):
        self.app.query_one("#select-download-path-container").remove_children()
        self.app.query_one("#select-download-path-container").mount(
            self.FilteredDirectoryTree(self.current_path, id="filteredtree")
        )
        self.app.query_one("#download-path-log").clear().write(
            shorten_path(self.current_path, 60)
        )

    def on_directory_tree_directory_selected(self, path):
        if self.current_path != str(get_downloads_folder()):
            self.current_path = str(path.path)
        self.refresh_tree()

    def on_button_pressed(self, event: Button.Pressed) -> None:
        if event.button.id == "download-path-back-button":
            if self.current_path == str(get_downloads_folder()):
                self.current_path = str(Path(get_data_drive_path()))
            else:
                self.current_path = str(
                    Path("\\".join(self.current_path.split("\\")[:-1]) + "\\")
                )
            self.refresh_tree()

        elif event.button.id == "download-path-reset-button":
            self.current_path = str(Path.cwd())
            self.refresh_tree()

        elif event.button.id == "downloads-folder-button":
            self.current_path = str(get_downloads_folder())
            self.refresh_tree()

        elif event.button.id == "download-path-confirm-button":
            self.dismiss(self.current_path)



from textual.widgets import RichLog, Button
from textual.containers import Container
from textual.screen import Screen
from textual.widgets import Header,Footer

from spotidex.cli.utils import  Path

from rich.table import Table
from rich.align import Align

from spotidex.cli.custom_widgets.AppInterface import AppInterface
from spotidex.cli.app_screens.SelectDownloadInterface import SelectDownload
from spotidex.cli.app_screens.DownloadInterface import Download



CSS = """

MainMenuInterface{
    align:center middle;
}

#main-menu-interface #interface{
    padding: 2 5;
    grid-rows:1fr 3;
    grid-gutter: 1 1;
    
}

#link-info-container{
    width: 84;
    border: heavy ansi_bright_magenta;
    border-title-align: center;
    content-align:center middle;
    padding-left:1;
}

#link-info{
    width: 86;
    background:transparent;
}

#download-button{
    width:100%;
    column-span:1;
}

#download-all-button{
    width:100%
}

#select-download-button{
    width:100%;
}

#settings-button{
    width:98%;
    column-span:1;
}


"""


class MainMenuInterface(Screen):
    DEFAULT_CSS = CSS

    def __init__(self) -> None:
        self.spotify_link = self.app.spotify_link
        self.link_details = self.app.link_details
        self.app.selected_tracks = []
        self.app.install_screen(SelectDownload, "SelectDownloadScreen")
        super().__init__()

    def compose(self):
        childrens = [
            Container(
                RichLog(
                    highlight=True,
                    markup=True,
                    id="link-info",
                    auto_scroll=False,
                ),
                id="link-info-container",
            ),
            Button("SETTINGS", "primary", id="settings-button"),
        ]
        if "track" not in self.spotify_link:
            childrens.insert(
                1, Button("DOWNLOAD ALL TRACK", "success", id="download-all-button")
            )
            childrens.insert(
                2, Button("SELECT TO DOWNLOAD", "success", id="select-download-button")
            )
        else:
            childrens.insert(
                1, Button("DOWNLOAD TRACK", "success", id="download-button")
            )

        yield AppInterface(*childrens, id="main-menu-interface")
  
    def on_mount(self):
        link_info = self.link_details[0]
        text_log = self.query_one("#link-info")

        if "track" in self.spotify_link:
            self.query_one("#link-info-container").border_title = "TRACK INFO"
        elif "playlist" in self.spotify_link:
            self.query_one("#link-info-container").border_title = "PLAYLIST INFO"
        else:
            self.query_one("#link-info-container").border_title = "ALBUM INFO"

        text_log.clear()
        table = Table(
            show_header=False,
            row_styles=["yellow", "yellow", "yellow"],
            width=80,
            show_lines=True,
        )

        for i in link_info:
            table.add_row(
                Align(i, align="center"),
                Align(
                    (
                        link_info[i][:60] + "..."
                        if len(link_info[i]) > 60
                        else link_info[i]
                    ),
                    align="center",
                ),
            )
        text_log.write(table)

        self.app.query_one("#nav-label").update("[bold]MAIN MENU")
        if "track" in self.spotify_link:
            self.query_one("#interface").styles.grid_size_columns = 2
            self.query_one("#link-info-container").styles.column_span = 2
            self.query_one("#interface").styles.grid_gutter_vertical = 3

        else:
            self.query_one("#interface").styles.grid_size_columns = 3
            self.query_one("#link-info-container").styles.column_span = 3
            self.query_one("#interface").styles.grid_gutter_vertical = 2

    def on_button_pressed(self, event: Button.Pressed) -> None:
        if event.button.id == "download-button":
            self.app.push_screen(Download())
        elif event.button.id == "download-all-button":
            self.app.push_screen(Download())
            self.app.selected_tracks = self.link_details[1].keys()
        elif event.button.id == "select-download-button":
            self.app.push_screen("SelectDownloadScreen")
        elif event.button.id == "settings-button":
            self.app.push_screen("SettingsScreen")

        self.app.mount(Header(show_clock=True))
        self.app.mount(Footer())



from textual.widgets import Label, Select
from textual.containers import Container
from textual.widget import Widget
from textual.reactive import reactive
from textual import on

CSS = """
DownloadQualitySelection{
    height:auto;
    width:auto;
}

#download-quality-container{
    layout:horizontal;
    width:55;
    height:5;
    background:$boost;}

#download-quality-selection{
    width: 33;
    margin-left:1;
    margin-top:1;}

#download-quality-label{
    background:$boost;
    height:5;
    content-align:center middle;
    padding:1;}

#download-quality-selection > SelectCurrent {
    border: tall $boost;}
"""


class DownloadQualitySelection(Widget):
    DEFAULT_CSS = CSS
    value = reactive("high")
    def compose(self):
        options = [
            ("LOW QUALITY    [24  kbps]","low"),
            ("MEDIUM QUALITY [128 kbps]","medium"),
            ("HIGH QUALITY   [320 kbps]","high"),
        ]
        with Container(id="download-quality-container"):
            yield Label(" DOWNLOAD QUALITY ", id="download-quality-label")
            yield Select(
                options,
                allow_blank=False,
                id="download-quality-selection",
                
            )
    def on_select_changed(self):
        self.value = self.query_one(Select).value
    def watch_value(self,value):
        self.query_one(Select).value = value

import sys

sys.path.append("D:/Coding Projects/gui")
from textual.app import App


class TestApp(App):
    TITLE = "TEST APP"

    def compose(self):
        yield DownloadQualitySelection()

if __name__ == "__main__":
    TestApp().run()



from textual.containers import Container
from textual.widgets import Static
from textual.widget import Widget
from textual.reactive import reactive

CSS = """
MetadataCheckBox{
    height:auto;
    width:auto;

}

#main-container{
    height:auto;
    width:auto;
    padding: 1 2;
    background:$boost;
}


#metadata-checkbox-container{
    layout:horizontal;
    width:auto;
    height:3;
    background:$boost;
}

#metadata-checkbox-label{
    width:auto;
    margin-top:1;
    margin-left:2;
}

#metadata-checkbox{
    border:thick transparent ;
    width:7;
    height:3;
    dock:right;
    padding-left:1;
    margin-left:1;
  
}

#metadata-checkbox:hover.inactive{
    opacity:0.6;

}

#metadata-checkbox:hover.active{
    opacity:1;

}

.inactive{
    background:#787878;
    opacity:0.5;
}
.active{
    background:$success 90%;
}

"""


class MetadataCheckBox(Widget):
    DEFAULT_CSS = CSS
    SCOPED_CSS = True
    value = reactive(True)

    def compose(self):
        with Container(id="main-container"):
            with Container(id="metadata-checkbox-container"):
                yield Static("INCLUDE METADATA", id="metadata-checkbox-label")
                yield Static(id="metadata-checkbox")

    def watch_value(self, value):
        if value == True:
            self.query_one("#metadata-checkbox").update("✔️")
            self.query_one("#metadata-checkbox").remove_class("inactive")
            self.query_one("#metadata-checkbox").add_class("active")
        else:
            self.query_one("#metadata-checkbox").update("")
            self.query_one("#metadata-checkbox").remove_class("active")
            self.query_one("#metadata-checkbox").add_class("inactive")
 

    def on_click(self) -> None:
        if self.value == True:
            self.value = False
        else:
            self.value = True
